/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.84a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_780066.c   (for NEC K0 780066 CPU and IAR compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include <intrinsics.h>
#include <io78f0066.h>

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU

  Select UART for embOSView, set baudrate,
  If you do not want (or can not due to hardware limitations) to dedicate
  a UART to embOSView, please define OS_UART to -1
*/

#ifndef   OS_FSYS
  #define OS_FSYS 4194304L          /* 78K - Test it ! board */
#endif
#define TIMER_RELOAD (OS_FSYS/1000) /* Default: 1000 Ticks / second */

#ifndef   OS_UART
  #define OS_UART 0                 /* Use UART0 per default        */
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 9600L
#endif

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for embOS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  /* timer settings */
  TMC0   = 0x00;           /* Stop and Clear Timer 0 */
  CRC0   = 0x00;           /* Register CR00, CR01 operate as compare register */
  CR00   = TIMER_RELOAD;   /* Compare values for 1ms Intervall */
  TOC0   = 0x00;           /* Disable Timer output */
  PRM0   = 0x00;           /* Clock selection: fx (4.914304MHz) */
  TMIF00 = 0;              /* Clear interrupt request Bit Timer0 */
  TMMK00 = 0;              /* Enable Timer0 Interrupt */
  TMC0   = 0x0C;           /* Start Timer 0 */
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {     /* Idle loop: No task is ready to execute  */
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via embOSView or high
*       resolution time management functions.
*       It returns the system time in timer clock cycles.
*/
#define HW_TIMER TM0
#define HW_TIMER_INTERRUPT_PENDING() (TMIF00)

OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 t_cnt   = HW_TIMER;
  OS_U32 time    = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = HW_TIMER;
    time++;
  }
  return TIMER_RELOAD * time  + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds. (For profiling only)
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling only.
*       It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*       Do not modify anything except interrupt vector
*/
#define OS_TICKVECT    INTTM00_vect

#pragma vector=OS_TICKVECT
__interrupt void OS_ISR_Tick (void);              /* Avoid warning */
__interrupt void OS_ISR_Tick (void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_TICK_Handle();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView via UART0 (optional)
*
**********************************************************************
*/

#if (OS_UART == 0)
  #if   (OS_BAUDRATE==38400) && (OS_FSYS==4194304)
    #define BRGC_VAL  0x1b
  #elif (OS_BAUDRATE==19200) && (OS_FSYS==4194304)
    #define BRGC_VAL  0x2b
  #elif (OS_BAUDRATE==9600)  && (OS_FSYS==4194304)
    #define BRGC_VAL  0x3b
  #else
    #error please specify BRGC_VAL
  #endif

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART0 for communication with embOSView
*/
  void OS_COM_Init(void) {
    P7     = 0x01;                     /* Set output latch to 0x01 (enable RS232 driver 78K - Testit) */
    PU7    = 0x08;                     /* Enable internal pull-up resistor P7.3 */
    PM7    = 0x88;                     /* Port 7.7,7.3 input, other output */
    BRGC0  = BRGC_VAL;                 /* Set calculated baudrate value */
    ASIM0  = 0xC8;                     /* UART transmit/receive mode */
                                       /* 1 stopbit, 8 data bit, no parity */
    SRIF0  = 0;                        /* Clear Interrupt request reception complete */
    STIF0  = 0;                        /* Clear Interrupt request transmision complete */
    SRMK0  = 0;                        /* Enable Interrupt reception complete */
    STMK0  = 0;                        /* Enable Interrupt transmision complete */
  }

/*********************************************************************
*
*       OS_COM_Send1()
*       CAUTION: Internal use, NEVER call this from your appication
*/
  void OS_COM_Send1(unsigned char c) {
    TXS0 = c;
  }

/*********************************************************************
*
*       OS_ISR_tx()
*/
  #pragma vector=INTST0_vect
  __interrupt void OS_ISR_tx(void);                /* Avoid warning */
  __interrupt void OS_ISR_tx(void) {
    OS_EnterInterrupt();
    OS_EnterIntStack();
    OS_OnTx();
    OS_LeaveIntStack();
    OS_LeaveInterrupt();
  }

/*********************************************************************
*
*       OS_ISR_rx()
*/
  #pragma vector=INTSR0_vect
  __interrupt void OS_ISR_rx(void);                /* Avoid warning */
  __interrupt void OS_ISR_rx(void) {
    OS_EnterNestableInterrupt();
    OS_EnterIntStack();
    if (ASIS0&0x07) {
      RXB0;                /* Dummy read, reset error, discard data */
    } else {
      OS_OnRx(RXB0);
    }
    OS_LeaveIntStack();
    OS_LeaveNestableInterrupt();
  }

/********************************************************************/

#else  /* selected UART not supported, using dummies */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);           /* Keep compiler happy */
    OS_COM_ClearTxActive();  /* Let OS know that transmitter is not busy  */
  }

#endif /*  (OS_UART == 0)  */

/*****  End Of File  ************************************************/

