/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.84a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_780148.c   (for NEC K0 780148 CPU and IAR compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"
#include <io78f0148.h>

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU

  Select UART for embOSView, set baudrate,
  If you do not want (or can not due to hardware limitations) to dedicate
  a UART to embOSView, please define OS_UART to -1
*/

#ifndef OS_FSYS
  #define OS_FSYS 8000000L
#endif
#define TIMER_RELOAD (OS_FSYS/1000) /* Default: 1000 Ticks / second */

#ifndef OS_UART
  #define OS_UART 6              /* 0 => UART0, 6 => UART6 selected */
#endif

#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 9600L
#endif

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for embOS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  TMC00   = 0x00;           /* Stop and Clear Timer 0 */
  CRC00   = 0x00;           /* Register CR00, CR01 operate as compare register */
  CR000   = TIMER_RELOAD;   /* Compare values for 1ms Interval */
  TOC00   = 0x00;           /* Disable Timer output */
  PRM00   = 0x00;           /* Clock selection: fx = OS_FSYS */
  TMIF000 = 0;              /* Clear interrupt request Bit Timer0 */
  TMMK000 = 0;              /* Enable Timer0 Interrupt */
  TMC00   = 0x0C;           /* Start Timer 0 */
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {     /* Idle loop: No task is ready to execute  */
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via embOSView or high
*       resolution time management functions.
*       It returns the system time in timer clock cycles.
*/
#define HW_TIMER TM00
#define HW_TIMER_INTERRUPT_PENDING() (TMIF000)
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 t_cnt   = HW_TIMER;
  OS_U32 time    = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = HW_TIMER;
    time++;
  }
  return TIMER_RELOAD * time  + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds. (For profiling only)
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling only.
*       It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*       Do not modify anything except interrupt vector
*/

#define OS_TICKVECT    INTTM000_vect

#pragma vector=OS_TICKVECT
__interrupt void OS_ISR_Tick (void); /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_Tick (void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_TICK_Handle();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView via UART0 (optional)
*
**********************************************************************
*/

#if (OS_UART == 0)
  #if (OS_BAUDRATE <= 9600)
    #define UART_PRESCALER 32
    #define TPSS_VAL 0xC0
  #else
    #define UART_PRESCALER 8
    #define TPSS_VAL 0x80
  #endif
  #define MDL_VAL (OS_FSYS / (2 * OS_BAUDRATE * UART_PRESCALER))
  #if ((MDL_VAL < 8) || (MDL_VAL > 31))
    #error "Selected Baudrate can not be set, please modify settings"
  #endif

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART0 for communication with embOSView
*/
  void OS_COM_Init(void) {
    SRMK0   =  1;       /* Disable Rx Interrupt                */
    DUALMK0 =  1;       /* Disable Tx Interrupt                */
    P1      =  (1<<0);  /* Set output latch P10 to enable TxD0 */
    PU1     =  (1<<0)   /* Enable internal pull-up on TxD0     */
              |(1<<1);  /* and RxD0                            */
    PM1    |=  (1<<1);  /* Port 11 (RxD0) set as input         */
    PM1    &= ~(1<<0);  /* Port 10 (TxD0) set as output        */
    ASIM0   =  0x81;    /* internal power on, rx tx disabled   */
    ASIM0   =  (1<<7)   /* Set data format                     */
              |(0<<3)   /* No parity                           */
              |(1<<2)   /* 8 data bits                         */
              |(0<<1)   /* 1 Stop bit                          */
              |(1<<1);  /* Always 1                            */
    BRGC0   =  TPSS_VAL | MDL_VAL; /* Set calculated baudrate  */
    ASIM0  |=  (1<<6);  /* Enable TxD                          */
    ASIM0  |=  (1<<5);  /* Enable RxD                          */
    DUALPR0 =  1;       /* Set Tx int to low priority          */
    SRPR0   =  1;       /* Set Rx Int to low priority          */
    SRIF0   =  0;       /* Clear Rx Interrupt                  */
    DUALIF0 =  0;       /* Clear Tx Interrupt                  */
    SRMK0   =  0;       /* Enable Rx Interrupt                 */
    DUALMK0 =  0;       /* Enable Tx Interrupt                 */
  }

/*********************************************************************
*
*       OS_COM_Send1()
*       CAUTION: Internal use, NEVER call this from your appication
*/
  void OS_COM_Send1(unsigned char c) {
    TXS0 = c;
  }

/*********************************************************************
*
*       OS_ISR_tx()
*/
  #pragma vector=INTST0_vect
  __interrupt void OS_ISR_tx(void);  /* Avoid "has no prototype" warning */
  __interrupt void OS_ISR_tx(void) {
    OS_EnterInterrupt();
    OS_EnterIntStack();
    OS_OnTx();
    OS_LeaveIntStack();
    OS_LeaveInterrupt();
  }

/*********************************************************************
*
*       OS_ISR_rx()
*/
  #pragma vector=INTSR0_vect
  __interrupt void OS_ISR_rx(void);  /* Avoid "has no prototype" warning */
  __interrupt void OS_ISR_rx(void) {
    OS_EnterNestableInterrupt();
    OS_EnterIntStack();
    if (ASIS0 & 0x07) {
      RXB0;        /*  Reset error, discard Byte */
    } else {
      OS_OnRx(RXB0);
    }
    OS_LeaveIntStack();
    OS_LeaveNestableInterrupt();
  }

/*********************************************************************
*
*       Communication for embOSView via UART6 (optional)
*
**********************************************************************
*/

#elif (OS_UART == 6)              /*  UART6 used for embOSView  */
  #define UART_PRESCALER 4
  #define BRGC_VAL  (OS_FSYS / (OS_BAUDRATE * 2 * UART_PRESCALER))
  #if ((BRGC_VAL < 8) || (BRGC_VAL > 255))
    #error "Please specify BRGC_VAL for UART6"
  #endif

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART6 for communication with embOSView
*/
  void OS_COM_Init(void) {
    SRMK6  =  1;       /* Disable Rx Interrupt                */
    STMK6  =  1;       /* Disable Tx Interrupt                */
    P1     =  (1<<3);  /* Set output latch P13 to enable TxD6 */
    PU1    =  (1<<3)   /* Enable internal pull-up on TxD6     */
             |(1<<4);  /* and RxD6                            */
    PM1   |=  (1<<4);  /* Port 14 (RxD6) set as input         */
    PM1   &= ~(1<<3);  /* Port 13 (TxD6) set as output        */
    #ifdef PLAY_IT     /* Demo kit needs Shutdown for RS232-  */
    P14    =  (1<<1);  /* driver on Port 14.3 passive         */
    PM14  &= ~(1<<1);  /* Enable output                       */
    P14   |=  (1<<1);  /* Shutdown set high (enable RS232)    */
    #endif
    ASIM6  =  0;       /* Power off to enble prescaler init   */
    CKSR6  =  0x02;    /* BRG clock = OS_FSYS/4               */
    ASIM6  =  (1<<7);  /* internal power on, rx tx disabled   */
    ASIM6  =  (1<<7)   /* Set data format                     */
             |(0<<3)   /* No parity                           */
             |(1<<2)   /* 8 data bits                         */
             |(0<<1)   /* 1 Stop bit                          */
             |(1<<1);  /* Rx Interrupt on Rx error            */
    BRGC6  =  BRGC_VAL;/* Set caculated baudrate value        */
    ASIM6 |=  (1<<6);  /* Enable TxD                          */
    ASIM6 |=  (1<<5);  /* Enable RxD                          */
    STPR6  =  1;       /* Set Tx int to low priority          */
    SRPR6  =  1;       /* Set Rx Int to low priority          */
    SRIF6  =  0;       /* Clear Rx Interrupt                  */
    STIF6  =  0;       /* Clear Tx Interrupt                  */
    SRMK6  =  0;       /* Enable Rx Interrupt                 */
    STMK6  =  0;       /* Enable Tx Interrupt                 */
  }

/*********************************************************************
*
*       OS_COM_Send1()
*       CAUTION: Internal use, NEVER call this from your application
*/
  void OS_COM_Send1(unsigned char c) {
    #ifdef PLAY_IT
    P7 ^= 0x40;
    #endif
    TXB6 = c;
  }

/*********************************************************************
*
*       OS_ISR_tx()
*/
  #pragma vector=INTST6_vect
  __interrupt void OS_ISR_tx(void);  /* Avoid "has no prototype" warning */
  __interrupt void OS_ISR_tx(void) {
    OS_EnterInterrupt();
    OS_EnterIntStack();
    OS_OnTx();
    #ifdef PLAY_IT
    P7 ^= 0x20;
    #endif
    OS_LeaveIntStack();
    OS_LeaveInterrupt();
  }

/*********************************************************************
*
*       OS_ISR_rx()
*/
  #pragma vector=INTSR6_vect
  __interrupt void OS_ISR_rx(void);  /* Avoid "has no prototype" warning */
  __interrupt void OS_ISR_rx(void) {
    OS_EnterNestableInterrupt();
    OS_EnterIntStack();
    if (ASIS6 & 0x07) {
      RXB6;               /*  Reset error, discard Byte */
      #ifdef PLAY_IT
      P7 ^= 0x40;
      #endif
    } else {
      #ifdef PLAY_IT
      P7 ^= 0x80;
      #endif
      OS_OnRx(RXB6);
    }
    OS_LeaveIntStack();
    OS_LeaveNestableInterrupt();
  }

/********************************************************************/

#else  /* selected UART not supported, using dummies */

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* avoid compiler warning              */
  OS_COM_ClearTxActive();  /* let embOS know that Tx is not busy  */
}

#endif /*  ((OS_UART == 0) || (OS_UART == 6))  */

/*****  End Of File  ************************************************/

