/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.84a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_78F1166.c  for NEC K0R 78F1166 CPU and IAR compiler
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include <io78f1166_a0.h>
#include <io78f1166_a0_ext.h>

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS
  #define OS_FSYS 20000000L
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER OS_FSYS     /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS      /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#define   TIMER_RELOAD (OS_PCLK_TIMER/1000)  /* 1000 Ticks / second */

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code does not enable a UART per default
*       and supports UART3 only
*/
#ifndef   OS_UART
  #define OS_UART (-1)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 9600L
#endif

/*********************************************************************
*
*       CPU configuration,
*       Option bytes and security code
*/
#pragma location = "OPTBYTE"
__root const unsigned char opbytes[3]={0x00,0xFF,0x85};

#pragma location = "SECUID"
__root const unsigned char secuid[10]={0xff,0xff,0xff,0xff,0xff,
                                       0xff,0xff,0xff,0xff,0xff};


/****** End of configuration settings *******************************/

#define OS_UART_USED (OS_UART == 3)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       __low_level_init()
*       Perform hardware initialization which has to be done
*       before memory is initialized
*/
int __low_level_init(void) {
  //
  // Speed up system clock
  //
  OSMC = 0x01;           // Operation speed mode control register
                         // Frequency higher than 10MHz
  OSTS = 0x07;           // Set osc. stabilization time selection
                         // to 2^18/fx
  CMC  = 0x51;           // Clock operation mode register
                         // X1 osc. mode, XT1 osc. mode, fx > 10MHz
  CKC  = 0x08;           // System clock control register
                         // fclk = fih
  CSC  = 0x00;           // enable X1 , XT1 operation
  while(OSTC < 0xFF) {   // Wait until fX1 clock stabilization
    __no_operation();    // time has expired
  }
  CKC  = 0x18;           // System clock control register
                         // fclk = fmx = 20MHz
  CSC  = 0x01;           // stop internal high speed oscillator
  return 1;              // always return 1 to enable segment initialization
}

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for embOS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  //
  // Initialy disable interrupts
  //
  OS_IncDI();
  //
  // Enable Timer array unit clock first
  // This is required before programming any other timer array sfr
  //
  TAU0EN = 1;
  //
  // Initialize Timer array 0, channel 0 in free running mode
  //
  TT0    &= ~(1uL << 0);    // Disable timer channel, required before modifying other registers
  TPS0   &= 0xFFF0uL;       // Select prescaler for clock 0, full speed
  TMR00  &= 0xFFF0uL;       // b3..b0  = 0 => Interval timer mode without interrupt on start
  TMR00  &= ~(7uL << 8);    // b10..b8 = 0 => Only software trigger is valid as trigger

  TDR00   = TIMER_RELOAD-1; /* Compare values for 1ms Interval */

  TS0    |= (1uL << 0);     // Start channel 0

  TMIF00  = 0;              /* Clear interrupt request Bit Timer0 */
  TMMK00  = 0;              /* Enable Timer0 Interrupt */

  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {                 /* Idle loop: No task is ready */
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via embOSView or high
*       resolution time management functions.
*       It returns the system time in timer clock cycles.
*/
#define HW_TIMER TCR00
#define HW_TIMER_INTERRUPT_PENDING() (TMIF00)
OS_U32 OS_GetTime_Cycles(void) {
  OS_U16 TCnt;
  OS_U32 Time;

  Time = OS_GetTime32() + 1;
  TCnt   = HW_TIMER;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    TCnt = HW_TIMER;
    Time++;
  }
  return TIMER_RELOAD * Time - TCnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds. (For profiling only)
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling only.
*       It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*       Do not modify anything except interrupt vector
*/
#define OS_TICKVECT   INTTM00_vect
#pragma vector=OS_TICKVECT
static __interrupt void OS_ISR_Tick (void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_TICK_Handle();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView via UART0 (optional)
*
**********************************************************************
*/
#if OS_UART_USED
//
// Map SFRs and interrupt functions to selected UART
//
#if (OS_UART == 3)
  #define OS_UART_TX_vect   INTST3_vect
  #define OS_UART_RX_vect   INTSR3_vect

  #define SDRrx             SDR13
  #define SDRtx             SDR12
  #define SAUxEN            SAU1EN
  #define SRMKx             SRMK3
  #define STMKx             STMK3
  #define SSm               SS1
  #define SSX_VAL           0x000C
  #define SMRtx             SMR12
  #define SMRrx             SMR13
  #define SCRtx             SCR12
  #define SCRrx             SCR13
  #define SOm               SO1
  #define SOEm              SOE1
  #define SPSm              SPS1
#endif

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART0 for communication with embOSView
*/
void OS_COM_Init(void) {
  SAUxEN = 1;                   // Enable peripheral clock for UART
  // After setting SAUxEN, wait for 4 clocks, before writing to SPS
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  //
  // Right now, only UART 3 is supported, the following code is
  // therefore specific to initialize UART 3
  //
  SPSm   = 0x0004;              // SAU initialization for transmit and receive mode
                                // Unit 1 channels 2 and 3 are used
  SMRtx  = 0x0022;              // Set unit 1 channel 2 to UART mode,
                                // Software trigger only (transmit mode)
  SMRrx  = 0x0122;              // Set unit 1 channel 3 to UART mode,
                                // Valid egde of RxD pin (receive mode)
  SDRtx  = 0x8000;              // Set clock divider of unit 1 channel 2
                                // Baudrate => 9600
  SDRrx  = 0x8000;              // Set clock divider of unit 1 channel 3
                                // Baudrate => 9600
  SCRtx  = 0x8097;              // Set transmit mode, no parity, LSB first,
                                // One stop bit, 8-bit data length
  SCRrx  = 0x4097;              // Set receive mode, no parity, LSB first,
                                // One stop bit, 8-bit data length
  SOm    = 0x0004;              // Set unit 1 Channel 2 initial data output is 0
  SOEm   = 0x0004;              // Set unit 1 Channel 2 output enable

  P1_bit.no3  = 1;              // Set TXD to 1
  PM1_bit.no3 = 0;              // Set TXD to output
  PU1_bit.no4 = 1;              // Enable RXD Pull-Up resistor
  PM1_bit.no4 = 1;              // Set RXD to input

  SSm   = SSX_VAL;              // Start serial unit, both channels
  SRMKx = 0;                    // Enable UART RxInterrupt
  STMKx = 0;                    // Enable TxInterrupt
}

/*********************************************************************
*
*       OS_COM_Send1()
*       CAUTION: Internal use, NEVER call this from your appication
*/
void OS_COM_Send1(unsigned char c) {
  SDRtx = c;           // Send char
}

/*********************************************************************
*
*       OS_ISR_tx()
*/
#pragma vector = OS_UART_TX_vect
static __interrupt void OS_ISR_tx(void) {
  OS_EnterInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx()
*/
#pragma vector = OS_UART_RX_vect
static __interrupt void OS_ISR_rx(void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_OnRx(SDRrx);
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/********************************************************************/

#else  /* selected UART not supported, using dummies */
void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* avoid compiler warning              */
  OS_COM_ClearTxActive();  /* let embOS know that Tx is not busy  */
}

#endif /*  OS_UART_USED  */

/*****  End Of File *************************************************/
